@@ -1,10 +1,12 @@
 from datetime import datetime
+from posixpath import split
 from pprint import pprint
 
 import bson
 
-from .init_creds import init_mongo
-from .dates import set_next_vigency, calculate_payment_date, get_periodo, localize, DIAS_ANTICIPO_DE_PAGO, today_argentina
+from .init_creds import init_mongo, test
+from .dates import set_next_vigency, calculate_payment_date, get_periodo, localize, DIAS_ANTICIPO_DE_PAGO, \
+    today_argentina
 from .format import datetime_parser
 from .mercadopago_ops import get_payments_from_user_id
 from dateutil.relativedelta import *
@@ -12,18 +14,23 @@ import mercadopago
 from bson import ObjectId
 
 
-def get_client_price(client: dict, plan_price: int, corpo_discount: float, sum_access=True) -> dict:
+def get_client_price(client: dict, plan_price: int, corpo_discount: float, sum_access=True,
+                     access_count_limit: int = 9999) -> dict:
     """ Obtener el precio a pagar
 
     :param client: cliente con la estructura que se usa en mongodb
     :type client: dict
-    :param plan: precio del plan nativo
-    :type plan: int
-    :param plan: porcentaje de descuento del plan corporativo
-    :type plan: float
+    :param plan_price: precio del plan nativo
+    :type plan_price: int
+    :param corpo_discount: porcentaje de descuento del plan corporativo
+    :type corpo_discount: float
+    :param sum_access: Si es True, suma los accesos desde el último cobro
+    :type sum_access: bool
+    :param access_count_limit: tope de cantidad de accesos pagos a cobrar
+    :type access_count_limit: int
     :return: informacion del precio
     :rtype: dict
-    """    
+    """
 
     prices = {"native_plan": plan_price}
     if corpo_discount:
@@ -37,7 +44,8 @@ def get_client_price(client: dict, plan_price: int, corpo_discount: float, sum_a
                         concepto = discount["concepto"]
                     else:
                         concepto = "n/a"
-                    prices["Descuento " + concepto] = - (plan_price * discount["porcentaje"]) /100 - discount["monto_absoluto"]
+                    prices["Descuento " + concepto] = - (plan_price * discount["porcentaje"]) / 100 - discount[
+                        "monto_absoluto"]
             except:
                 print(f"    ERROR en el cálculo de descuentos - se retorna un diccionario de precios vacío.")
                 return {}
@@ -45,7 +53,7 @@ def get_client_price(client: dict, plan_price: int, corpo_discount: float, sum_a
     subtotal = max(0, sum(prices.values()))
 
     if sum_access:
-        accesos = get_access_amount(client["_id"])
+        accesos = get_access_amount(client["_id"], max_count=access_count_limit)
         prices["access"] = accesos
     else:
         accesos = 0
@@ -55,11 +63,13 @@ def get_client_price(client: dict, plan_price: int, corpo_discount: float, sum_a
     return prices
 
 
-def get_access_amount(c_id: bson.ObjectId) -> int:
+def get_access_amount(c_id: bson.ObjectId, max_count: int = 9999) -> int:
     """ Obtener monto de acceso
 
     :param c_id: id del socio
     :type c_id: bson.ObjectId
+    :param max_count: Límite de cantidad de accesos con costo a cobrar.
+    :type max_count: int
     :return: Monto de acceso
     :rtype: int
     """
@@ -69,7 +79,7 @@ def get_access_amount(c_id: bson.ObjectId) -> int:
         last_payment_date = max(last_payment)
     else:
         last_payment_date = datetime(year=2000, month=1, day=1)
-    uses = db.accesos.find({"socio_id": c_id, "fecha": {"$gt": last_payment_date}, "costo": {"$gt": 0}})
+    uses = db.accesos.find({"socio_id": c_id, "date": {"$gt": last_payment_date}, "costo": {"$gt": 0}}).limit(max_count)
     result = sum([a["costo"] for a in uses])
     print(f"    El monto de accesos a cobrar es de {result} acumulados desde {last_payment_date}.")
     return result
@@ -80,7 +90,7 @@ def substract_discounts(client: dict):
 
     :param client: cliente con la estructura que se usa en mongodb
     :type client: dict
-    """        
+    """
     if client["discounts"]:
         for discount in client["discounts"]:
             if discount["aplicaciones_restantes"] > 0:
@@ -108,18 +118,23 @@ def resolve_payment(payment_data: dict, card_data: dict, _client: dict, _plan: d
     :return: informacion con el resultado del proceso
     :rtype: dict
     """
+
+    if _client["preferred_payment_method"] == "efectivo":
+        payment_result = {
+            "status": "pending_efectivo"
+        }
+        return payment_result
+
     if not payment_data["charges_detail"]:
         return {"status": "error",
                 "status_detail": "Error al calcular el precio del cliente.",
-                "prev_status": payment_data["status"],
                 "date_created": today_argentina()}
 
     if payment_data["charges_detail"]["final_price"] > 0:
-        if card_data["card_id"] is None:
+        if card_data["id"] is None:
             print("    Tarjeta no encontrada. No se puede procesar el pago.")
             payment_result = {"status": "error",
                               "status_detail": "No se puede obtener la tarjeta activa.",
-                              "prev_status": payment_data["status"],
                               "date_created": today_argentina()}
         else:
             print("    Se procede a realizar un pago de $" + str(payment_data["charges_detail"]["final_price"]))
@@ -132,7 +147,6 @@ def resolve_payment(payment_data: dict, card_data: dict, _client: dict, _plan: d
             "status": "approved",
             "status_detail": "Pago no realizado por ser precio 0.",
             "id": 11,
-            "prev_status": payment_data["status"],
             "date_created": today_argentina()
         }
     return payment_result
@@ -154,9 +168,9 @@ def process_payment(payment_data: dict, card_data: dict, _client: dict, _plan: d
     :return: informacion con el resultado del proceso
     :rtype: dict
     """
-        
+
     print("    Se procede a realizar el pago con MP")
-    data = {"card_id": int(card_data["card_id"])}
+    data = {"card_id": int(card_data["id"])}
     card_token_response = sdk.card_token().create(data)
 
     if card_token_response["status"] > 299:
@@ -165,39 +179,92 @@ def process_payment(payment_data: dict, card_data: dict, _client: dict, _plan: d
         return {"status": "error",
                 "status_detail": "Falló la creación de token de la tarjeta.",
                 "response": card_token_response["response"]["message"],
-                "prev_status": payment_data["status"],
                 "date_created": today_argentina()
                 }
 
     card_token = card_token_response["response"]["id"]
     print(f"        Token generado OK")
-    
+
     if "tries" in payment_data.keys():
         n_try = len(payment_data["tries"]) + 1
         numero_intento = "Reintento " + str(n_try)
     else:
         numero_intento = "Primer intento"
-    
+
+    try:
+        street_number = int(_client["domicilio"]["altura"])
+    except:
+        street_number = None
     mp_payment_data = {
         "additional_info": {
             "items": [{
-                "title": f'{_plan["name"]} - {_client["nombre"]} {_client["apellido"]} - {_client["email"]} -' 
+                "id": str(_plan['_id']),
+                "title": f'{_plan["name"]} - {_client["nombre"]} {_client["apellido"]} - {_client["email"]} -'
                          f'{payment_data["period"]}',
                 "category_id": _plan["nivel_de_acceso"],
-                "description": numero_intento +"- id: " + f'{_client["_id"]}'
-            }]
+                "description": numero_intento + "- id: " + f'{_client["_id"]}'
+            }],
+            "shipments": {
+                "receiver_address": {
+                    "state_name": _client["domicilio"]["provincia"],
+                    "city_name": _client["domicilio"]["localidad"],
+                    "street_name": _client["domicilio"]["calle"],
+                    "street_number": street_number,
+                }
+            },
+            "payer": {
+                "first_name": _client["nombre"],
+                "last_name": _client["apellido"],
+                "address": None
+            }
         },
-        "notification_url": f"https://apisportclub.xyz/notificacion/mp"
-                            f"?source_news=webhooks&merchant={str(_plan['merchant_id'])}",
+        "notification_url": None if test else f"https://apisportclub.xyz/notificacion/mp"
+                                              f"?source_news=webhooks&merchant={str(_plan['merchant_id'])}",
         "transaction_amount": round(payment_data["charges_detail"]["final_price"], 2),
         "token": card_token,  # ??
         "description": _plan["name"],  # ir a base de datos de planes y traer el name
         "installments": 1,  # ??
         "payer": {
-            "id": _client["mercadopago_id"]
+            "first_name": _client["nombre"],
+            "last_name": _client["apellido"],
+            "id": _client["mercadopago_id"],
+            "address": None
         },
         "external_reference": str(payment_data["_id"])
     }
+
+    point_of_interaction = {}
+
+    if "poi" in _client.keys():
+        aux = False
+        number = _client["poi"]["installments"] + 1
+        subscription_secuence = str(_client["poi"]["payment_reference"])
+        point_of_interaction["transaction_data"] = {"payment_reference": {"id":subscription_secuence}}
+
+    else:
+        aux = True
+        number = 1
+        point_of_interaction["transaction_data"] = {}
+
+    point_of_interaction["transaction_data"].update({
+            "first_time_use": aux,
+            "subscription_sequence": {
+                "number": number,
+                "total": 32
+            },
+            "invoice_period": {
+                "period": number,
+                "type": "monthly"
+            },
+            "billing_date": today_argentina().strftime("%Y-%m-%d")
+    })
+    point_of_interaction["type"] = "SUBSCRIPTIONS"
+    mp_payment_data["point_of_interaction"] = point_of_interaction
+
+    if "seller_fee" in payment_data.keys():
+        if 100 > payment_data["seller_fee"] > 0:
+            mp_payment_data["application_fee"] = round((1 - payment_data["seller_fee"]/100) * payment_data["charges_detail"]["final_price"],2)
+
     payment_attempt = sdk.payment().create(mp_payment_data)
     payment_response = datetime_parser(payment_attempt["response"])
 
@@ -214,7 +281,6 @@ def process_payment(payment_data: dict, card_data: dict, _client: dict, _plan: d
             "status": "error",
             "status_detail": detail,
             "response": payment_attempt["response"],
-            "prev_status": payment_data["status"],
             "date_created": today_argentina()
         }
 
@@ -223,12 +289,11 @@ def process_payment(payment_data: dict, card_data: dict, _client: dict, _plan: d
         "status": payment_response["status"],
         "status_detail": payment_response["status_detail"],
         "id": payment_response["id"],
-        "prev_status": payment_data["status"],
         "date_created": today_argentina()
     }
 
 
-def create_payment_data(_client: dict, prices: dict, merchant_id: ObjectId, source: str) -> dict:
+def create_payment_data(_client: dict, prices: dict, merchant_id: ObjectId, source: str, seller:dict=None, insert=True, id_operacion=None) -> dict:
     """ Crear data del pago y guardalo en la coleccion de boletas de mongodb
 
     :param _client: cliente con la estructura que se usa en mongodb
@@ -239,12 +304,14 @@ def create_payment_data(_client: dict, prices: dict, merchant_id: ObjectId, sour
     :type merchant_id: ObjectId
     :param source: de dónde proviene la boleta - checkout o recurring_charges
     :type source: str
+    :param insert: si es True, se inserta boleta en base de datos
+    :type source: bool
     :return: data que se guardo en la coleccion de boletas
     :rtype: dict
     """
     db = init_mongo()
 
-    if "checkout" in source:
+    if "checkout" in source or "cobro_aon" in source:
         opd = today_argentina()
         period = get_periodo(opd)
     else:
@@ -252,18 +319,26 @@ def create_payment_data(_client: dict, prices: dict, merchant_id: ObjectId, sour
         period = get_periodo(opd + relativedelta(days=DIAS_ANTICIPO_DE_PAGO))
 
     data = {"member_id": _client["_id"],
-            "date_created": today_argentina(),
-            "original_payment_date": opd,
-            "source": source,
-            "tries": [],
-            "status": "pending",
-            "merchant_id": merchant_id,
-            "charges_detail": prices,
-            "period": period,
-            "plan_id": _client["active_plan_id"]}
+        "date_created": today_argentina(),
+        "original_payment_date": opd,
+        "source": source,
+        "tries": [],
+        "status": "error_not_processed",
+        "merchant_id": merchant_id,
+        "charges_detail": prices,
+        "period": period,
+        "plan_id": _client["active_plan_id"]
+    }
+    if seller:
+        data["seller_fee"] = seller["fee"]
+        data["seller_merchant_id"] = seller["merchant_id"]
 
-    _id = db.boletas.insert_one(data)
-    print(f"    Se inserta una nueva boleta de pago con _id: {_id.inserted_id}.")
+    if 'cobro_aon' in source and id_operacion != None:
+        data['id_operacion'] = id_operacion
+
+    if insert:
+        _id = db.boletas.insert_one(data)
+        print(f"    Se inserta una nueva boleta de pago con _id: {_id.inserted_id}")
 
     return data
 
@@ -288,7 +363,7 @@ def update_payment_data(payment_result: dict, payment_data: dict, card_data: dic
         "payment_day": payment_result["date_created"],
         "payment_type": card_data["card_type"],
         "card_brand": card_data["card_brand"],
-        "card_id": card_data["card_id"],
+        "card_id": card_data["id"],
         "status": payment_result["status"],
         "status_detail": payment_result["status_detail"],
         "payment_id": payment_result["id"]
@@ -298,6 +373,55 @@ def update_payment_data(payment_result: dict, payment_data: dict, card_data: dic
     return
 
 
+def format_card(mp_card):
+    if set(mp_card.keys()) == {'id', 'payer_id', 'card_type', 'card_brand', 'last_four_digits', 'issuer',
+                               'cardholder', 'expiration_month', 'expiration_year'}:
+        return mp_card
+    try:
+        return {
+            'id': mp_card['id'],
+            'card_type': mp_card['payment_method']['payment_type_id'],
+            'card_brand': mp_card["payment_method"]["name"],
+            'last_four_digits': mp_card['last_four_digits'],
+            'issuer': {
+                'name': mp_card['issuer']['name']
+            },
+            'cardholder': {
+                'name': mp_card['cardholder']['name']
+            },
+            'expiration_month': mp_card['expiration_month'],
+            'expiration_year': mp_card['expiration_year'],
+            'payer_id': mp_card['user_id']
+        }
+    except KeyError:
+        return mp_card
+
+
+def get_empty_card():
+    return {
+        "id": None,
+        "payer_id": None,
+        "card_type": None,
+        "card_brand": None
+    }
+
+
+def get_cards(client: dict):
+    cards = []
+    active_card = client['active_card'] if 'active_card' in client else None
+
+    for card in client['cards']:
+        if 'status' in card and card['status'] == 'invalid':
+            continue
+        formatted_card = format_card(card)
+        if formatted_card['id'] == active_card:
+            cards.insert(0, formatted_card)
+        else:
+            cards.append(formatted_card)
+
+    return cards
+
+
 def get_active_card(_client: dict, price: dict, sdk: mercadopago.SDK):
     """ Obtener la tarjeta activa (con la que se realizan los pagos)
 
@@ -311,10 +435,7 @@ def get_active_card(_client: dict, price: dict, sdk: mercadopago.SDK):
     :rtype: dict
     """
 
-    card = {"card_id": None,
-            "payer_id": None,
-            "card_type": None,
-            "card_brand": None}
+    card = get_empty_card()
     if not price:
         print("    No existe precio - se devuelve tarjeta vacía")
         return card
@@ -332,7 +453,7 @@ def get_active_card(_client: dict, price: dict, sdk: mercadopago.SDK):
                 else:
                     card = _client["cards"][0]
                 print("    Se obtiene tarjeta correctamente.")
-                return {"card_id": card["id"],
+                return {"id": card["id"],
                         "payer_id": card["customer_id"],
                         "card_type": card["payment_method"]["payment_type_id"],
                         "card_brand": card["payment_method"]["name"]}
@@ -342,22 +463,22 @@ def get_active_card(_client: dict, price: dict, sdk: mercadopago.SDK):
         else:
             print("         Cliente sin atributo 'cards'.")
 
-    if not card["card_id"] and "active_card" in _client.keys() and "mercadopago_id" in _client.keys():
+    if not card["id"] and "active_card" in _client.keys() and "mercadopago_id" in _client.keys():
         mp_card_data = sdk.card().get(_client["mercadopago_id"], _client["active_card"])
         if mp_card_data["status"] == 200:
             mp_card_data = mp_card_data["response"]
-            card["card_id"] = mp_card_data["id"]
+            card["id"] = mp_card_data["id"]
             card["payer_id"] = mp_card_data["user_id"]
             card["card_type"] = mp_card_data["payment_method"]["payment_type_id"]
             card["card_brand"] = mp_card_data["payment_method"]["name"]
             print("         Se reconstruyó tarjeta a partir de MP exitosamente.")
         elif "active_card" in _client.keys():
-            card["card_id"] = _client["active_card"]
+            card["id"] = _client["active_card"]
 
     return card
 
 
-def update_client(_client: dict, _payment_result: dict, _plan: dict, period: str):
+def update_client(_client: dict, _payment_result: dict, _plan: dict, period: str, update_npd: bool = False):
     """ Actualizar cliente
 
     :param _client: Cliente con la estructura que se usa en mongodb
@@ -368,6 +489,8 @@ def update_client(_client: dict, _payment_result: dict, _plan: dict, period: str
     :type _plan: dict
     :param period: periodo de la última boleta generada
     :type period: str
+    :param update_npd: indica si se debe actualizar el next_payment_date del cliente
+    :type update_npd: bool
     """
     # si el pago tiene id_mp se los agrego al cliente
     if "id" in _payment_result.keys():
@@ -378,7 +501,7 @@ def update_client(_client: dict, _payment_result: dict, _plan: dict, period: str
     # actualizaciones de cliente
     _client["lastModified"] = today_argentina()
 
-    if _payment_result["prev_status"] == "pending":
+    if update_npd:
         _client["next_payment_date"] = calculate_payment_date(_client["period_init_day"], _plan["cobro"], period)
         print("         El próximo dia de cobro es el " + _client["next_payment_date"].strftime("%d/%m/%y"))
         # se resta una aplicacion de descuento
@@ -390,11 +513,17 @@ def update_client(_client: dict, _payment_result: dict, _plan: dict, period: str
         _client["cobros_recurrentes"] += 1
         if _client["next_payment_date"]:
             _client["fecha_vigencia"] = set_next_vigency(_client["next_payment_date"])
+            print("         Se actualiza la fecha de vigencia: {}".format(_client["fecha_vigencia"]))
+
+        if "poi" in _client.keys():
+            _client["poi"]["installments"] += 1
+        else:
+            _client["poi"] = {"installments": 1, "payment_reference": _payment_result["id"]}
+
     else:
         if localize(_client["fecha_vigencia"]) < today_argentina():
             _client["status"] = "inactivo"
             print("         El cliente pasa inactivo por vencer la vigencia")
-
     return
 
 
@@ -408,8 +537,8 @@ def restore_pending_payment(user_id, boleta_id):
     """
     db = init_mongo()
     boleta = db.boletas.find_one({"_id": ObjectId(boleta_id)})
-    if boleta["status"] != "pending":
-        print("Boleta no estaba pendiente")
+    if boleta["status"] != "error_not_processed":
+        print("Boleta no estaba rota")
         return
     client = db.clientes.find_one({"_id": ObjectId(user_id)})
     plan = db.planes.find_one({"_id": client["active_plan_id"]})
@@ -423,10 +552,9 @@ def restore_pending_payment(user_id, boleta_id):
         "status": payment_response["status"],
         "status_detail": payment_response["status_detail"],
         "id": payment_response["id"],
-        "prev_status": boleta["status"],
-        "date_created":  payment_response["date_created"]
+        "date_created": payment_response["date_created"]
     }
-    card = {"card_id": payment_response["card"]["id"],
+    card = {"id": payment_response["card"]["id"],
             "payer_id": payment_response["card"]["cardholder"]["identification"]["number"],
             "card_type": payment_response["payment_type_id"],
             "card_brand": payment_response["payment_method_id"]}
@@ -472,7 +600,6 @@ def build_payment_result(mp_payment: dict) -> dict:
     pr = {
         "status": mp_payment["status"],
         "status_detail": mp_payment["status_detail"],
-        "prev_status": "pending",
         "id": mp_payment["id"],
         "date_created": mp_payment["date_created"]
     }
@@ -481,8 +608,27 @@ def build_payment_result(mp_payment: dict) -> dict:
 
 def get_card_data_from_payment(mp_payment: dict) -> dict:
     return {
-            "card_id": mp_payment["card"]["id"],
-            "payer_id": mp_payment["payer"]["id"],
-            "card_type": mp_payment["payment_type_id"],
-            "card_brand": mp_payment["payment_method_id"]
-            }
\ No newline at end of file
+        "id": mp_payment["card"]["id"],
+        "payer_id": mp_payment["payer"]["id"],
+        "card_type": mp_payment["payment_type_id"],
+        "card_brand": mp_payment["payment_method_id"]
+    }
+
+
+def get_other_cards(_client, card_data):
+    active_card_id = card_data["id"]
+    cards_list = []
+    for card in _client["cards"]:
+        if "id" in card.keys():
+            if card["id"]:
+                if active_card_id == card["id"]:
+                    continue
+                else:
+                    card_data = {
+                        "card_id": card["id"],
+                        "payer_id": card["customer_id"],
+                        "card_type": card["payment_method"]["payment_type_id"],
+                        "card_brand": card["payment_method"]["name"]
+                    }
+                    cards_list.append(card_data)
+    return cards_list
