@@ -4,12 +4,14 @@ proj_accesso_cliente = {'token_spice': 1,
                                "fecha_vigencia": 1,
                                "nivel_de_acceso": 1,
                                "status": 1,
-                               "sede": 1,
+
                                "nombre": 1,
                                "apellido": 1,
                                "nacimiento": 1,
                                 "active_plan_id": 1,
                         "email": 1,
                         "documento": 1,
-                        "plan_corporativo": 1
+                        "plan_corporativo": 1,
+                        "plan": 1,
+                        "sede": 1
                         }
\ No newline at end of file
